@@ -46,7 +46,6 @@ many of the difficult problems for you, making the task of building
 sophisticated high-performance network servers and clients a snap.
 """
 
-import exceptions
 import select
 import socket
 import sys
@@ -54,14 +53,14 @@ import time
 
 import os
 from errno import EALREADY, EINPROGRESS, EWOULDBLOCK, ECONNRESET, \
-     ENOTCONN, ESHUTDOWN, EINTR, EISCONN
+     ENOTCONN, ESHUTDOWN, EINTR, EISCONN, errorcode
 
 try:
     socket_map
 except NameError:
     socket_map = {}
 
-class ExitNow(exceptions.Exception):
+class ExitNow(Exception):
     pass
 
 def read(obj):
@@ -80,12 +79,22 @@ def write(obj):
     except:
         obj.handle_error()
 
+def _exception (obj):
+    try:
+        obj.handle_expt_event()
+    except ExitNow:
+        raise
+    except:
+        obj.handle_error()
+
 def readwrite(obj, flags):
     try:
-        if flags & select.POLLIN:
+        if flags & (select.POLLIN | select.POLLPRI):
             obj.handle_read_event()
         if flags & select.POLLOUT:
             obj.handle_write_event()
+        if flags & (select.POLLERR | select.POLLHUP | select.POLLNVAL):
+            obj.handle_expt_event()
     except ExitNow:
         raise
     except:
@@ -97,10 +106,14 @@ def poll(timeout=0.0, map=None):
     if map:
         r = []; w = []; e = []
         for fd, obj in map.items():
-            if obj.readable():
+            is_r = obj.readable()
+            is_w = obj.writable()
+            if is_r:
                 r.append(fd)
-            if obj.writable():
+            if is_w:
                 w.append(fd)
+            if is_r or is_w:
+                e.append(fd)
         if [] == r == w == e:
             time.sleep(timeout)
         else:
@@ -124,31 +137,13 @@ def poll(timeout=0.0, map=None):
                 continue
             write(obj)
 
-def poll2(timeout=0.0, map=None):
-    import poll
-    if map is None:
-        map = socket_map
-    if timeout is not None:
-        # timeout is in milliseconds
-        timeout = int(timeout*1000)
-    if map:
-        l = []
-        for fd, obj in map.items():
-            flags = 0
-            if obj.readable():
-                flags = poll.POLLIN
-            if obj.writable():
-                flags = flags | poll.POLLOUT
-            if flags:
-                l.append((fd, flags))
-        r = poll.poll(l, timeout)
-        for fd, flags in r:
+        for fd in e:
             obj = map.get(fd)
             if obj is None:
                 continue
-            readwrite(obj, flags)
+            _exception(obj)
 
-def poll3(timeout=0.0, map=None):
+def poll2(timeout=0.0, map=None):
     # Use the poll() support added to the select module in Python 2.0
     if map is None:
         map = socket_map
@@ -160,10 +155,13 @@ def poll3(timeout=0.0, map=None):
         for fd, obj in map.items():
             flags = 0
             if obj.readable():
-                flags = select.POLLIN
+                flags |= select.POLLIN | select.POLLPRI
             if obj.writable():
-                flags = flags | select.POLLOUT
+                flags |= select.POLLOUT
             if flags:
+                # Only check for exceptions if object was either readable
+                # or writable.
+                flags |= select.POLLERR | select.POLLHUP | select.POLLNVAL
                 pollster.register(fd, flags)
         try:
             r = pollster.poll(timeout)
@@ -177,35 +175,45 @@ def poll3(timeout=0.0, map=None):
                 continue
             readwrite(obj, flags)
 
-def loop(timeout=30.0, use_poll=0, map=None):
+poll3 = poll2                           # Alias for backward compatibility
+
+def loop(timeout=30.0, use_poll=False, map=None, count=None):
     if map is None:
         map = socket_map
 
-    if use_poll:
-        if hasattr(select, 'poll'):
-            poll_fun = poll3
-        else:
-            poll_fun = poll2
+    if use_poll and hasattr(select, 'poll'):
+        poll_fun = poll2
     else:
         poll_fun = poll
 
-    while map:
-        poll_fun(timeout, map)
+    if count is None:
+        while map:
+            poll_fun(timeout, map)
+
+    else:
+        while map and count > 0:
+            poll_fun(timeout, map)
+            count = count - 1
 
 class dispatcher:
 
-    debug = 0
-    connected = 0
-    accepting = 0
-    closing = 0
+    debug = False
+    connected = False
+    accepting = False
+    closing = False
     addr = None
 
     def __init__(self, sock=None, map=None):
+        if map is None:
+            self._map = socket_map
+        else:
+            self._map = map
+
         if sock:
             self.set_socket(sock, map)
             # I think it should inherit this anyway
             self.socket.setblocking(0)
-            self.connected = 1
+            self.connected = True
             # XXX Does the constructor require that the socket passed
             # be connected?
             try:
@@ -232,16 +240,17 @@ class dispatcher:
     def add_channel(self, map=None):
         #self.log_info('adding channel %s' % self)
         if map is None:
-            map = socket_map
+            map = self._map
         map[self._fileno] = self
 
     def del_channel(self, map=None):
         fd = self._fileno
         if map is None:
-            map = socket_map
+            map = self._map
         if map.has_key(fd):
             #self.log_info('closing channel %d:%s' % (fd, self))
             del map[fd]
+        self._fileno = None
 
     def create_socket(self, family, type):
         self.family_and_type = family, type
@@ -276,21 +285,15 @@ class dispatcher:
     def readable(self):
         return True
 
-    if os.name == 'mac':
-        # The macintosh will select a listening socket for
-        # write if you let it.  What might this mean?
-        def writable(self):
-            return not self.accepting
-    else:
-        def writable(self):
-            return True
+    def writable(self):
+        return True
 
     # ==================================================
     # socket object methods.
     # ==================================================
 
     def listen(self, num):
-        self.accepting = 1
+        self.accepting = True
         if os.name == 'nt' and num > 5:
             num = 1
         return self.socket.listen(num)
@@ -300,17 +303,17 @@ class dispatcher:
         return self.socket.bind(addr)
 
     def connect(self, address):
-        self.connected = 0
+        self.connected = False
         err = self.socket.connect_ex(address)
         # XXX Should interpret Winsock return values
         if err in (EINPROGRESS, EALREADY, EWOULDBLOCK):
             return
         if err in (0, EISCONN):
             self.addr = address
-            self.connected = 1
+            self.connected = True
             self.handle_connect()
         else:
-            raise socket.error, err
+            raise socket.error, (err, errorcode[err])
 
     def accept(self):
         # XXX can return either an address pair or None
@@ -321,7 +324,7 @@ class dispatcher:
             if why[0] == EWOULDBLOCK:
                 pass
             else:
-                raise socket.error, why
+                raise
 
     def send(self, data):
         try:
@@ -331,7 +334,7 @@ class dispatcher:
             if why[0] == EWOULDBLOCK:
                 return 0
             else:
-                raise socket.error, why
+                raise
             return 0
 
     def recv(self, buffer_size):
@@ -350,7 +353,7 @@ class dispatcher:
                 self.handle_close()
                 return ''
             else:
-                raise socket.error, why
+                raise
 
     def close(self):
         self.del_channel()
@@ -377,11 +380,11 @@ class dispatcher:
             # for an accepting socket, getting a read implies
             # that we are connected
             if not self.connected:
-                self.connected = 1
+                self.connected = True
             self.handle_accept()
         elif not self.connected:
             self.handle_connect()
-            self.connected = 1
+            self.connected = True
             self.handle_read()
         else:
             self.handle_read()
@@ -390,7 +393,7 @@ class dispatcher:
         # getting a write implies that we are connected
         if not self.connected:
             self.handle_connect()
-            self.connected = 1
+            self.connected = True
         self.handle_write()
 
     def handle_expt_event(self):
@@ -442,8 +445,8 @@ class dispatcher:
 
 class dispatcher_with_send(dispatcher):
 
-    def __init__(self, sock=None):
-        dispatcher.__init__(self, sock)
+    def __init__(self, sock=None, map=None):
+        dispatcher.__init__(self, sock, map)
         self.out_buffer = ''
 
     def initiate_send(self):
@@ -497,7 +500,7 @@ def close_all(map=None):
 #
 # After a little research (reading man pages on various unixen, and
 # digging through the linux kernel), I've determined that select()
-# isn't meant for doing doing asynchronous file i/o.
+# isn't meant for doing asynchronous file i/o.
 # Heartening, though - reading linux/mm/filemap.c shows that linux
 # supports asynchronous read-ahead.  So _MOST_ of the time, the data
 # will be sitting in memory for us already when we go to read it.
@@ -526,21 +529,21 @@ if os.name == 'posix':
         write = send
 
         def close(self):
-            return os.close(self.fd)
+            os.close(self.fd)
 
         def fileno(self):
             return self.fd
 
     class file_dispatcher(dispatcher):
 
-        def __init__(self, fd):
-            dispatcher.__init__(self)
-            self.connected = 1
+        def __init__(self, fd, map=None):
+            dispatcher.__init__(self, None, map)
+            self.connected = True
+            self.set_file(fd)
             # set it to non-blocking mode
             flags = fcntl.fcntl(fd, fcntl.F_GETFL, 0)
             flags = flags | os.O_NONBLOCK
             fcntl.fcntl(fd, fcntl.F_SETFL, flags)
-            self.set_file(fd)
 
         def set_file(self, fd):
             self._fileno = fd
