@@ -5,26 +5,17 @@
 
 
 
-#ifdef _WIN32
-#include "pywintoolbox.h"
-#else
-#include "macglue.h"
 #include "pymactoolbox.h"
-#endif
 
 /* Macro to test whether a weak-loaded CFM function exists */
 #define PyMac_PRECHECK(rtn) do { if ( &rtn == NULL )  {\
-    	PyErr_SetString(PyExc_NotImplementedError, \
-    	"Not available in this shared library/OS version"); \
-    	return NULL; \
+        PyErr_SetString(PyExc_NotImplementedError, \
+        "Not available in this shared library/OS version"); \
+        return NULL; \
     }} while(0)
 
 
-#ifdef WITHOUT_FRAMEWORKS
-#include <TextEdit.h>
-#else
 #include <Carbon/Carbon.h>
-#endif
 
 #ifdef USE_TOOLBOX_OBJECT_GLUE
 extern PyObject *_TEObj_New(TEHandle);
@@ -44,21 +35,21 @@ static PyObject *
 TextStyle_New(TextStylePtr itself)
 {
 
-	return Py_BuildValue("lllO&", (long)itself->tsFont, (long)itself->tsFace, (long)itself->tsSize, QdRGB_New,
-				&itself->tsColor);
+        return Py_BuildValue("lllO&", (long)itself->tsFont, (long)itself->tsFace, (long)itself->tsSize, QdRGB_New,
+                                &itself->tsColor);
 }
 
 static int
 TextStyle_Convert(PyObject *v, TextStylePtr p_itself)
 {
-	long font, face, size;
-	
-	if( !PyArg_ParseTuple(v, "lllO&", &font, &face, &size, QdRGB_Convert, &p_itself->tsColor) )
-		return 0;
-	p_itself->tsFont = (short)font;
-	p_itself->tsFace = (Style)face;
-	p_itself->tsSize = (short)size;
-	return 1;
+        long font, face, size;
+
+        if( !PyArg_ParseTuple(v, "lllO&", &font, &face, &size, QdRGB_Convert, &p_itself->tsColor) )
+                return 0;
+        p_itself->tsFont = (short)font;
+        p_itself->tsFace = (Style)face;
+        p_itself->tsSize = (short)size;
+        return 1;
 }
 
 static PyObject *TE_Error;
@@ -67,7 +58,7 @@ static PyObject *TE_Error;
 
 PyTypeObject TE_Type;
 
-#define TEObj_Check(x) ((x)->ob_type == &TE_Type)
+#define TEObj_Check(x) ((x)->ob_type == &TE_Type || PyObject_TypeCheck((x), &TE_Type))
 
 typedef struct TEObject {
 	PyObject_HEAD
@@ -78,14 +69,15 @@ PyObject *TEObj_New(TEHandle itself)
 {
 	TEObject *it;
 	if (itself == NULL) {
-						PyErr_SetString(TE_Error,"Cannot create null TE");
-						return NULL;
-					}
+	                                PyErr_SetString(TE_Error,"Cannot create null TE");
+	                                return NULL;
+	                        }
 	it = PyObject_NEW(TEObject, &TE_Type);
 	if (it == NULL) return NULL;
 	it->ob_itself = itself;
 	return (PyObject *)it;
 }
+
 int TEObj_Convert(PyObject *v, TEHandle *p_itself)
 {
 	if (!TEObj_Check(v))
@@ -100,7 +92,7 @@ int TEObj_Convert(PyObject *v, TEHandle *p_itself)
 static void TEObj_dealloc(TEObject *self)
 {
 	TEDispose(self->ob_itself);
-	PyMem_DEL(self);
+	self->ob_type->tp_free((PyObject *)self);
 }
 
 static PyObject *TEObj_TESetText(TEObject *_self, PyObject *_args)
@@ -774,134 +766,238 @@ static PyObject *TEObj_as_Resource(TEObject *_self, PyObject *_args)
 
 static PyMethodDef TEObj_methods[] = {
 	{"TESetText", (PyCFunction)TEObj_TESetText, 1,
-	 "(Buffer text) -> None"},
+	 PyDoc_STR("(Buffer text) -> None")},
 	{"TEGetText", (PyCFunction)TEObj_TEGetText, 1,
-	 "() -> (CharsHandle _rv)"},
+	 PyDoc_STR("() -> (CharsHandle _rv)")},
 	{"TEIdle", (PyCFunction)TEObj_TEIdle, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TESetSelect", (PyCFunction)TEObj_TESetSelect, 1,
-	 "(long selStart, long selEnd) -> None"},
+	 PyDoc_STR("(long selStart, long selEnd) -> None")},
 	{"TEActivate", (PyCFunction)TEObj_TEActivate, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEDeactivate", (PyCFunction)TEObj_TEDeactivate, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEKey", (PyCFunction)TEObj_TEKey, 1,
-	 "(CharParameter key) -> None"},
+	 PyDoc_STR("(CharParameter key) -> None")},
 	{"TECut", (PyCFunction)TEObj_TECut, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TECopy", (PyCFunction)TEObj_TECopy, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEPaste", (PyCFunction)TEObj_TEPaste, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEDelete", (PyCFunction)TEObj_TEDelete, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEInsert", (PyCFunction)TEObj_TEInsert, 1,
-	 "(Buffer text) -> None"},
+	 PyDoc_STR("(Buffer text) -> None")},
 	{"TESetAlignment", (PyCFunction)TEObj_TESetAlignment, 1,
-	 "(short just) -> None"},
+	 PyDoc_STR("(short just) -> None")},
 	{"TEUpdate", (PyCFunction)TEObj_TEUpdate, 1,
-	 "(Rect rUpdate) -> None"},
+	 PyDoc_STR("(Rect rUpdate) -> None")},
 	{"TEScroll", (PyCFunction)TEObj_TEScroll, 1,
-	 "(short dh, short dv) -> None"},
+	 PyDoc_STR("(short dh, short dv) -> None")},
 	{"TESelView", (PyCFunction)TEObj_TESelView, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEPinScroll", (PyCFunction)TEObj_TEPinScroll, 1,
-	 "(short dh, short dv) -> None"},
+	 PyDoc_STR("(short dh, short dv) -> None")},
 	{"TEAutoView", (PyCFunction)TEObj_TEAutoView, 1,
-	 "(Boolean fAuto) -> None"},
+	 PyDoc_STR("(Boolean fAuto) -> None")},
 	{"TECalText", (PyCFunction)TEObj_TECalText, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEGetOffset", (PyCFunction)TEObj_TEGetOffset, 1,
-	 "(Point pt) -> (short _rv)"},
+	 PyDoc_STR("(Point pt) -> (short _rv)")},
 	{"TEGetPoint", (PyCFunction)TEObj_TEGetPoint, 1,
-	 "(short offset) -> (Point _rv)"},
+	 PyDoc_STR("(short offset) -> (Point _rv)")},
 	{"TEClick", (PyCFunction)TEObj_TEClick, 1,
-	 "(Point pt, Boolean fExtend) -> None"},
+	 PyDoc_STR("(Point pt, Boolean fExtend) -> None")},
 	{"TESetStyleHandle", (PyCFunction)TEObj_TESetStyleHandle, 1,
-	 "(TEStyleHandle theHandle) -> None"},
+	 PyDoc_STR("(TEStyleHandle theHandle) -> None")},
 	{"TEGetStyleHandle", (PyCFunction)TEObj_TEGetStyleHandle, 1,
-	 "() -> (TEStyleHandle _rv)"},
+	 PyDoc_STR("() -> (TEStyleHandle _rv)")},
 	{"TEGetStyle", (PyCFunction)TEObj_TEGetStyle, 1,
-	 "(short offset) -> (TextStyle theStyle, short lineHeight, short fontAscent)"},
+	 PyDoc_STR("(short offset) -> (TextStyle theStyle, short lineHeight, short fontAscent)")},
 	{"TEStylePaste", (PyCFunction)TEObj_TEStylePaste, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TESetStyle", (PyCFunction)TEObj_TESetStyle, 1,
-	 "(short mode, TextStyle newStyle, Boolean fRedraw) -> None"},
+	 PyDoc_STR("(short mode, TextStyle newStyle, Boolean fRedraw) -> None")},
 	{"TEReplaceStyle", (PyCFunction)TEObj_TEReplaceStyle, 1,
-	 "(short mode, TextStyle oldStyle, TextStyle newStyle, Boolean fRedraw) -> None"},
+	 PyDoc_STR("(short mode, TextStyle oldStyle, TextStyle newStyle, Boolean fRedraw) -> None")},
 	{"TEGetStyleScrapHandle", (PyCFunction)TEObj_TEGetStyleScrapHandle, 1,
-	 "() -> (StScrpHandle _rv)"},
+	 PyDoc_STR("() -> (StScrpHandle _rv)")},
 	{"TEStyleInsert", (PyCFunction)TEObj_TEStyleInsert, 1,
-	 "(Buffer text, StScrpHandle hST) -> None"},
+	 PyDoc_STR("(Buffer text, StScrpHandle hST) -> None")},
 	{"TEGetHeight", (PyCFunction)TEObj_TEGetHeight, 1,
-	 "(long endLine, long startLine) -> (long _rv)"},
+	 PyDoc_STR("(long endLine, long startLine) -> (long _rv)")},
 	{"TEContinuousStyle", (PyCFunction)TEObj_TEContinuousStyle, 1,
-	 "(short mode, TextStyle aStyle) -> (Boolean _rv, short mode, TextStyle aStyle)"},
+	 PyDoc_STR("(short mode, TextStyle aStyle) -> (Boolean _rv, short mode, TextStyle aStyle)")},
 	{"TEUseStyleScrap", (PyCFunction)TEObj_TEUseStyleScrap, 1,
-	 "(long rangeStart, long rangeEnd, StScrpHandle newStyles, Boolean fRedraw) -> None"},
+	 PyDoc_STR("(long rangeStart, long rangeEnd, StScrpHandle newStyles, Boolean fRedraw) -> None")},
 	{"TENumStyles", (PyCFunction)TEObj_TENumStyles, 1,
-	 "(long rangeStart, long rangeEnd) -> (long _rv)"},
+	 PyDoc_STR("(long rangeStart, long rangeEnd) -> (long _rv)")},
 	{"TEFeatureFlag", (PyCFunction)TEObj_TEFeatureFlag, 1,
-	 "(short feature, short action) -> (short _rv)"},
+	 PyDoc_STR("(short feature, short action) -> (short _rv)")},
 	{"TEGetHiliteRgn", (PyCFunction)TEObj_TEGetHiliteRgn, 1,
-	 "(RgnHandle region) -> None"},
+	 PyDoc_STR("(RgnHandle region) -> None")},
 	{"as_Resource", (PyCFunction)TEObj_as_Resource, 1,
-	 "() -> (Handle _rv)"},
+	 PyDoc_STR("() -> (Handle _rv)")},
 	{NULL, NULL, 0}
 };
 
-PyMethodChain TEObj_chain = { TEObj_methods, NULL };
-
-static PyObject *TEObj_getattr(TEObject *self, char *name)
+static PyObject *TEObj_get_destRect(TEObject *self, void *closure)
 {
-
-				if( strcmp(name, "destRect") == 0 )
-					return Py_BuildValue("O&", PyMac_BuildRect,
-							&(*self->ob_itself)->destRect);
-				if( strcmp(name, "viewRect") == 0 )
-					return Py_BuildValue("O&", PyMac_BuildRect,
-							&(*self->ob_itself)->viewRect);
-				if( strcmp(name, "selRect") == 0 )
-					return Py_BuildValue("O&", PyMac_BuildRect,
-							&(*self->ob_itself)->selRect);
-				if( strcmp(name, "lineHeight") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->lineHeight);
-				if( strcmp(name, "fontAscent") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->fontAscent);
-				if( strcmp(name, "selPoint") == 0 )
-					return Py_BuildValue("O&", PyMac_BuildPoint,
-							(*self->ob_itself)->selPoint);
-				if( strcmp(name, "selStart") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->selStart);
-				if( strcmp(name, "selEnd") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->selEnd);
-				if( strcmp(name, "active") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->active);
-				if( strcmp(name, "just") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->just);
-				if( strcmp(name, "teLength") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->teLength);
-				if( strcmp(name, "txFont") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->txFont);
-				if( strcmp(name, "txFace") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->txFace);
-				if( strcmp(name, "txMode") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->txMode);
-				if( strcmp(name, "txSize") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->txSize);
-				if( strcmp(name, "nLines") == 0 )
-					return Py_BuildValue("h", (*self->ob_itself)->nLines);
-			
-	return Py_FindMethodInChain(&TEObj_chain, (PyObject *)self, name);
+	return Py_BuildValue("O&", PyMac_BuildRect, &(*self->ob_itself)->destRect);
 }
 
-#define TEObj_setattr NULL
+#define TEObj_set_destRect NULL
+
+static PyObject *TEObj_get_viewRect(TEObject *self, void *closure)
+{
+	return Py_BuildValue("O&", PyMac_BuildRect, &(*self->ob_itself)->viewRect);
+}
+
+#define TEObj_set_viewRect NULL
+
+static PyObject *TEObj_get_selRect(TEObject *self, void *closure)
+{
+	return Py_BuildValue("O&", PyMac_BuildRect, &(*self->ob_itself)->selRect);
+}
+
+#define TEObj_set_selRect NULL
+
+static PyObject *TEObj_get_lineHeight(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->lineHeight);
+}
+
+#define TEObj_set_lineHeight NULL
+
+static PyObject *TEObj_get_fontAscent(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->fontAscent);
+}
+
+#define TEObj_set_fontAscent NULL
+
+static PyObject *TEObj_get_selPoint(TEObject *self, void *closure)
+{
+	return Py_BuildValue("O&", PyMac_BuildPoint, (*self->ob_itself)->selPoint);
+}
+
+#define TEObj_set_selPoint NULL
+
+static PyObject *TEObj_get_selStart(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->selStart);
+}
+
+#define TEObj_set_selStart NULL
+
+static PyObject *TEObj_get_selEnd(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->selEnd);
+}
+
+#define TEObj_set_selEnd NULL
+
+static PyObject *TEObj_get_active(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->active);
+}
+
+#define TEObj_set_active NULL
+
+static PyObject *TEObj_get_just(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->just);
+}
+
+#define TEObj_set_just NULL
+
+static PyObject *TEObj_get_teLength(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->teLength);
+}
+
+#define TEObj_set_teLength NULL
+
+static PyObject *TEObj_get_txFont(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->txFont);
+}
+
+#define TEObj_set_txFont NULL
+
+static PyObject *TEObj_get_txFace(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->txFace);
+}
+
+#define TEObj_set_txFace NULL
+
+static PyObject *TEObj_get_txMode(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->txMode);
+}
+
+#define TEObj_set_txMode NULL
+
+static PyObject *TEObj_get_txSize(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->txSize);
+}
+
+#define TEObj_set_txSize NULL
+
+static PyObject *TEObj_get_nLines(TEObject *self, void *closure)
+{
+	return Py_BuildValue("h", (*self->ob_itself)->nLines);
+}
+
+#define TEObj_set_nLines NULL
+
+static PyGetSetDef TEObj_getsetlist[] = {
+	{"destRect", (getter)TEObj_get_destRect, (setter)TEObj_set_destRect, "Destination rectangle"},
+	{"viewRect", (getter)TEObj_get_viewRect, (setter)TEObj_set_viewRect, "Viewing rectangle"},
+	{"selRect", (getter)TEObj_get_selRect, (setter)TEObj_set_selRect, "Selection rectangle"},
+	{"lineHeight", (getter)TEObj_get_lineHeight, (setter)TEObj_set_lineHeight, "Height of a line"},
+	{"fontAscent", (getter)TEObj_get_fontAscent, (setter)TEObj_set_fontAscent, "Ascent of a line"},
+	{"selPoint", (getter)TEObj_get_selPoint, (setter)TEObj_set_selPoint, "Selection Point"},
+	{"selStart", (getter)TEObj_get_selStart, (setter)TEObj_set_selStart, "Start of selection"},
+	{"selEnd", (getter)TEObj_get_selEnd, (setter)TEObj_set_selEnd, "End of selection"},
+	{"active", (getter)TEObj_get_active, (setter)TEObj_set_active, "TBD"},
+	{"just", (getter)TEObj_get_just, (setter)TEObj_set_just, "Justification"},
+	{"teLength", (getter)TEObj_get_teLength, (setter)TEObj_set_teLength, "TBD"},
+	{"txFont", (getter)TEObj_get_txFont, (setter)TEObj_set_txFont, "Current font"},
+	{"txFace", (getter)TEObj_get_txFace, (setter)TEObj_set_txFace, "Current font variant"},
+	{"txMode", (getter)TEObj_get_txMode, (setter)TEObj_set_txMode, "Current text-drawing mode"},
+	{"txSize", (getter)TEObj_get_txSize, (setter)TEObj_set_txSize, "Current font size"},
+	{"nLines", (getter)TEObj_get_nLines, (setter)TEObj_set_nLines, "TBD"},
+	{NULL, NULL, NULL, NULL},
+};
+
 
 #define TEObj_compare NULL
 
 #define TEObj_repr NULL
 
 #define TEObj_hash NULL
+#define TEObj_tp_init 0
+
+#define TEObj_tp_alloc PyType_GenericAlloc
+
+static PyObject *TEObj_tp_new(PyTypeObject *type, PyObject *_args, PyObject *_kwds)
+{
+	PyObject *_self;
+	TEHandle itself;
+	char *kw[] = {"itself", 0};
+
+	if (!PyArg_ParseTupleAndKeywords(_args, _kwds, "O&", kw, TEObj_Convert, &itself)) return NULL;
+	if ((_self = type->tp_alloc(type, 0)) == NULL) return NULL;
+	((TEObject *)_self)->ob_itself = itself;
+	return _self;
+}
+
+#define TEObj_tp_free PyObject_Del
+
 
 PyTypeObject TE_Type = {
 	PyObject_HEAD_INIT(NULL)
@@ -912,14 +1008,39 @@ PyTypeObject TE_Type = {
 	/* methods */
 	(destructor) TEObj_dealloc, /*tp_dealloc*/
 	0, /*tp_print*/
-	(getattrfunc) TEObj_getattr, /*tp_getattr*/
-	(setattrfunc) TEObj_setattr, /*tp_setattr*/
+	(getattrfunc)0, /*tp_getattr*/
+	(setattrfunc)0, /*tp_setattr*/
 	(cmpfunc) TEObj_compare, /*tp_compare*/
 	(reprfunc) TEObj_repr, /*tp_repr*/
 	(PyNumberMethods *)0, /* tp_as_number */
 	(PySequenceMethods *)0, /* tp_as_sequence */
 	(PyMappingMethods *)0, /* tp_as_mapping */
 	(hashfunc) TEObj_hash, /*tp_hash*/
+	0, /*tp_call*/
+	0, /*tp_str*/
+	PyObject_GenericGetAttr, /*tp_getattro*/
+	PyObject_GenericSetAttr, /*tp_setattro */
+	0, /*tp_as_buffer*/
+	Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /* tp_flags */
+	0, /*tp_doc*/
+	0, /*tp_traverse*/
+	0, /*tp_clear*/
+	0, /*tp_richcompare*/
+	0, /*tp_weaklistoffset*/
+	0, /*tp_iter*/
+	0, /*tp_iternext*/
+	TEObj_methods, /* tp_methods */
+	0, /*tp_members*/
+	TEObj_getsetlist, /*tp_getset*/
+	0, /*tp_base*/
+	0, /*tp_dict*/
+	0, /*tp_descr_get*/
+	0, /*tp_descr_set*/
+	0, /*tp_dictoffset*/
+	TEObj_tp_init, /* tp_init */
+	TEObj_tp_alloc, /* tp_alloc */
+	TEObj_tp_new, /* tp_new */
+	TEObj_tp_free, /* tp_free */
 };
 
 /* ----------------------- End object type TE ----------------------- */
@@ -1068,8 +1189,6 @@ static PyObject *TE_TEToScrap(PyObject *_self, PyObject *_args)
 	return _res;
 }
 
-#if TARGET_API_MAC_CARBON
-
 static PyObject *TE_TEGetScrapHandle(PyObject *_self, PyObject *_args)
 {
 	PyObject *_res = NULL;
@@ -1084,9 +1203,6 @@ static PyObject *TE_TEGetScrapHandle(PyObject *_self, PyObject *_args)
 	                     ResObj_New, _rv);
 	return _res;
 }
-#endif
-
-#if TARGET_API_MAC_CARBON
 
 static PyObject *TE_TESetScrapHandle(PyObject *_self, PyObject *_args)
 {
@@ -1103,7 +1219,6 @@ static PyObject *TE_TESetScrapHandle(PyObject *_self, PyObject *_args)
 	_res = Py_None;
 	return _res;
 }
-#endif
 
 static PyObject *TE_LMGetWordRedraw(PyObject *_self, PyObject *_args)
 {
@@ -1155,37 +1270,31 @@ static PyObject *TE_as_TE(PyObject *_self, PyObject *_args)
 
 static PyMethodDef TE_methods[] = {
 	{"TEScrapHandle", (PyCFunction)TE_TEScrapHandle, 1,
-	 "() -> (Handle _rv)"},
+	 PyDoc_STR("() -> (Handle _rv)")},
 	{"TEGetScrapLength", (PyCFunction)TE_TEGetScrapLength, 1,
-	 "() -> (long _rv)"},
+	 PyDoc_STR("() -> (long _rv)")},
 	{"TENew", (PyCFunction)TE_TENew, 1,
-	 "(Rect destRect, Rect viewRect) -> (TEHandle _rv)"},
+	 PyDoc_STR("(Rect destRect, Rect viewRect) -> (TEHandle _rv)")},
 	{"TETextBox", (PyCFunction)TE_TETextBox, 1,
-	 "(Buffer text, Rect box, short just) -> None"},
+	 PyDoc_STR("(Buffer text, Rect box, short just) -> None")},
 	{"TEStyleNew", (PyCFunction)TE_TEStyleNew, 1,
-	 "(Rect destRect, Rect viewRect) -> (TEHandle _rv)"},
+	 PyDoc_STR("(Rect destRect, Rect viewRect) -> (TEHandle _rv)")},
 	{"TESetScrapLength", (PyCFunction)TE_TESetScrapLength, 1,
-	 "(long length) -> None"},
+	 PyDoc_STR("(long length) -> None")},
 	{"TEFromScrap", (PyCFunction)TE_TEFromScrap, 1,
-	 "() -> None"},
+	 PyDoc_STR("() -> None")},
 	{"TEToScrap", (PyCFunction)TE_TEToScrap, 1,
-	 "() -> None"},
-
-#if TARGET_API_MAC_CARBON
+	 PyDoc_STR("() -> None")},
 	{"TEGetScrapHandle", (PyCFunction)TE_TEGetScrapHandle, 1,
-	 "() -> (Handle _rv)"},
-#endif
-
-#if TARGET_API_MAC_CARBON
+	 PyDoc_STR("() -> (Handle _rv)")},
 	{"TESetScrapHandle", (PyCFunction)TE_TESetScrapHandle, 1,
-	 "(Handle value) -> None"},
-#endif
+	 PyDoc_STR("(Handle value) -> None")},
 	{"LMGetWordRedraw", (PyCFunction)TE_LMGetWordRedraw, 1,
-	 "() -> (UInt8 _rv)"},
+	 PyDoc_STR("() -> (UInt8 _rv)")},
 	{"LMSetWordRedraw", (PyCFunction)TE_LMSetWordRedraw, 1,
-	 "(UInt8 value) -> None"},
+	 PyDoc_STR("(UInt8 value) -> None")},
 	{"as_TE", (PyCFunction)TE_as_TE, 1,
-	 "(Handle h) -> (TEHandle _rv)"},
+	 PyDoc_STR("(Handle h) -> (TEHandle _rv)")},
 	{NULL, NULL, 0}
 };
 
@@ -1199,8 +1308,8 @@ void init_TE(void)
 
 
 
-		PyMac_INIT_TOOLBOX_OBJECT_NEW(TEHandle, TEObj_New);
-		PyMac_INIT_TOOLBOX_OBJECT_CONVERT(TEHandle, TEObj_Convert);
+	        PyMac_INIT_TOOLBOX_OBJECT_NEW(TEHandle, TEObj_New);
+	        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(TEHandle, TEObj_Convert);
 
 
 	m = Py_InitModule("_TE", TE_methods);
@@ -1210,9 +1319,12 @@ void init_TE(void)
 	    PyDict_SetItemString(d, "Error", TE_Error) != 0)
 		return;
 	TE_Type.ob_type = &PyType_Type;
+	if (PyType_Ready(&TE_Type) < 0) return;
 	Py_INCREF(&TE_Type);
-	if (PyDict_SetItemString(d, "TEType", (PyObject *)&TE_Type) != 0)
-		Py_FatalError("can't initialize TEType");
+	PyModule_AddObject(m, "TE", (PyObject *)&TE_Type);
+	/* Backward-compatible name */
+	Py_INCREF(&TE_Type);
+	PyModule_AddObject(m, "TEType", (PyObject *)&TE_Type);
 }
 
 /* ========================= End module _TE ========================= */
